/**
 * y25 - Visualizing Graphs in 2.5 Dimensions (2005)
 * 
 * Arbeitsbereich: Paralleles Rechnen Prof. Kaufmann
 * Studienarbeit:  Werner Jainek
 * Betreuung:      Nina Lehmann
 *
 */
package y25.base;

/**
 * Provides access to data associated with an edge. A edge map can be considered
 * as a map that allows only edges as keys. The edges used as keys for the
 * getter/setter methods must belong to the same layered graph.
 * <p>
 * The values bound to the edges are initialized with Java(TM) default
 * values(null, 0, 0.0, false) upon initialization.
 * <p>
 * This interface exists primarily because the original interface
 * {@link y.base.EdgeMap y.base.EdgeMap} only allows edges that belong to the
 * same graph as keys. This restriction is not posed by this interface. When
 * working with layered graphs, there might be a situation where one EdgeMap
 * should store information about edges from different graphs (wich all belong
 * to the same layered graph) - and this is where this interface comes into
 * play.
 * <p>
 * There is also another reason why this interface exists: orthogonality. The
 * newly defined interfaces {@link GraphMap} and {@link LayerEdgeMap} both have
 * a <code>remove</code> method, which is not true for the original
 * {@link y.base.NodeMap y.base.NodeMap} or
 * {@link y.base.EdgeMap y.base.EdgeMap}.
 * 
 * @author Werner Jainek
 */
public interface EdgeMap extends y.base.EdgeMap {

	/**
	 * Removes the value stored for the given edge, if present. Returns the
	 * value that was previously stored for the graph, or <code>null</code> if
	 * there was no mapping for that edge. A <code>null</code> return can also
	 * mean that the map previously associated <code>null</code> with the
	 * given edge.
	 */
	public Object remove(Object edge);

	/**
	 * Returns the value bound to the given edge. The value must have been
	 * assiciated to the given edge by a call to
	 * {@link EdgeMap#set(Object, Object) set}.
	 * 
	 * Returns <code>null</code> if the map contains no mapping for this key.
	 * A return value of <code>null</code> does not necessarily indicate that
	 * the map contains no mapping for the key; it is also possible that the map
	 * explicitly maps the key to <code>null</code>.
	 */
	public Object get(Object edge);

	/**
	 * Returns the boolean value bound to the given edge. The value must have
	 * been assiciated to the given edge by a call to
	 * {@link EdgeMap#setBool(Object, boolean) setBool}.
	 * 
	 * Returns <code>false</code> if the map contains no mapping for this key.
	 * A return value of <code>false</code> does not necessarily indicate that
	 * the map contains no mapping for the key; it is also possible that the map
	 * explicitly maps the key to <code>false</code>.
	 */
	public boolean getBool(Object edge);

	/**
	 * Returns the double value bound to the given edge. The value must have
	 * been assiciated to the given edge by a call to
	 * {@link EdgeMap#setDouble(Object, double) setDouble}.
	 * 
	 * Returns <code>0.0</code> if the map contains no mapping for this key. A
	 * return value of <code>0.0</code> does not necessarily indicate that the
	 * map contains no mapping for the key; it is also possible that the map
	 * explicitly maps the key to <code>0.0</code>.
	 */
	public double getDouble(Object edge);

	/**
	 * Returns the integer value bound to the given edge. The value must have
	 * been assiciated to the given edge by a call to
	 * {@link EdgeMap#setInt(Object, int) setInt}.
	 * 
	 * Returns <code>0</code> if the map contains no mapping for this key. A
	 * return value of <code>0</code> does not necessarily indicate that the
	 * map contains no mapping for the key; it is also possible that the map
	 * explicitly maps the key to <code>0</code>.
	 */
	public int getInt(Object edge);

	/**
	 * Associates the given value to the given edge.
	 */
	public void set(Object edge, Object value);

	/**
	 * Associates the given boolean value to the given edge.
	 */
	public void setBool(Object edge, boolean value);

	/**
	 * Associates the given double value to the given edge.
	 */
	public void setDouble(Object edge, double value);

	/**
	 * Associates the given integer value to the given edge.
	 */
	public void setInt(Object edge, int value);
}
