/**
 * y25 - Visualizing Graphs in 2.5 Dimensions (2005)
 * 
 * Arbeitsbereich: Paralleles Rechnen Prof. Kaufmann
 * Studienarbeit:  Werner Jainek
 * Betreuung:      Nina Lehmann
 *
 */
package y25.base;

import y.base.Graph;
import y.base.YCursor;

/**
 * A cursor interface for interating over graphs.
 * <p>
 * The typical use in conjunction with a LayeredGraph instance is as follows:
 * 
 * <pre>
 * LayeredGraph lg; // a layered graph
 * for (GraphCursor gc = lg.graphs(); gc.ok(); gc.next()) {
 * 	Graph g = gc.graph();
 * 	// ... do something with the current graph ...
 * }
 * </pre>
 * 
 * @author Werner Jainek
 */
public interface GraphCursor extends YCursor {

	/**
	 * Returns the graph at the current location of the cursor.
	 * 
	 * @return The graph currently pointed to by this cursor.
	 */
	public Graph graph();

	/**
	 * Moves the cursor to the cyclic next element of the underlying sequence.
	 * This is the next element if available, else it is the first element.
	 */
	public void cyclicNext();

	/**
	 * Moves the cursor to the cyclic previous element of the underlying
	 * sequence. This is the previous element if available, else it is the last
	 * element.
	 */
	public void cyclicPrev();

}
