/**
 * y25 - Visualizing Graphs in 2.5 Dimensions (2005)
 * 
 * Arbeitsbereich: Paralleles Rechnen Prof. Kaufmann
 * Studienarbeit:  Werner Jainek
 * Betreuung:      Nina Lehmann
 *
 */
package y25.base;

import java.util.Iterator;

import y.base.DataProvider;
import y.base.Graph;
import y.base.YCursor;
import y.base.YList;

/**
 * Specialized list implementation for instances of type
 * {@link y.base.Graph Graph}.
 * <p>
 * Iterating over the elements of this list is accomplished by retrieving a
 * <code>GraphCursor</code> via the method {@link #graphs graphs} (see
 * {@linkplain GraphCursor the GraphCursor documentation} for an example).
 * 
 * @author Werner Jainek
 */
public class GraphList extends YList {

	/**
	 * Creates an empty graph list.
	 */
	public GraphList() {
		super();
	}

	/**
	 * Creates a list that is initialized with a single graph provided.
	 */
	public GraphList(Graph graph) {
		super();
		add(graph);
	}

	/**
	 * Creates a list that is initialized with the graphs provided by the given
	 * array of graphs.
	 */
	public GraphList(Graph[] graphs) {
		super(graphs);
	}

	/**
	 * Creates a list that is initialized with the graphs provided by the given
	 * <code>GraphCursor</code> object.
	 */
	public GraphList(GraphCursor gc) {
		super(gc);
	}

	/**
	 * Creates a list that is initialized with those graphs from the given
	 * GraphCursor object for which the given data provider returns
	 * <code>true</code> upon calling its
	 * {@link y.base.DataProvider#getBool getBool} method.
	 * 
	 * @param gc
	 *            A graph cursor providing graphs that should be added to this
	 *            list.
	 * @param predicate
	 *            A data provider that acts as a inclusion predicate for each
	 *            graph accessible by the given graph cursor.
	 */
	public GraphList(GraphCursor gc, DataProvider predicate) {
		super(gc, predicate);
	}

	/**
	 * Creates a list that is initialized with the elements provided by the
	 * given Iterator object.
	 */
	public GraphList(Iterator it) {
		super(it);
	}

	/**
	 * Returns the first graph in this list, or <code>null</code> when the
	 * list is empty.
	 */
	public Graph firstGraph() {
		if (!isEmpty()) {
			return (Graph) first();
		} else
			return null;
	}

	/**
	 * Returns the last graph in this list, or <code>null</code> when the list
	 * is empty.
	 */
	public Graph lastGraph() {
		if (!isEmpty()) {
			return (Graph) last();
		} else
			return null;
	}

	/**
	 * Returns a graph cursor for the graphs.
	 */
	public GraphCursor graphs() {
		return new GraphCursorImpl(cursor());
	}

	/**
	 * An implementation of the <code>GraphCursor</code> interface based on a
	 * <code>YCursor</code>.
	 * 
	 * @author Werner Jainek
	 * 
	 */
	private class GraphCursorImpl implements GraphCursor {
		private YCursor cursor;

		public GraphCursorImpl(YCursor cur) {
			cursor = cur;
		}

		public Graph graph() {
			return (Graph) current();
		}

		public void cyclicNext() {
			next();
			if (!ok())
				toFirst();
		}

		public void cyclicPrev() {
			prev();
			if (!ok())
				toLast();
		}

		public boolean ok() {
			return cursor.ok();
		}

		public void next() {
			cursor.next();
		}

		public void prev() {
			cursor.prev();
		}

		public void toFirst() {
			cursor.toFirst();
		}

		public void toLast() {
			cursor.toLast();
		}

		public Object current() {
			return cursor.current();
		}

		public int size() {
			return cursor.size();
		}
	}
}
