/**
 * y25 - Visualizing Graphs in 2.5 Dimensions (2005)
 * 
 * Arbeitsbereich: Paralleles Rechnen Prof. Kaufmann
 * Studienarbeit:  Werner Jainek
 * Betreuung:      Nina Lehmann
 *
 */
package y25.base;

import java.util.Iterator;

import y.base.DataProvider;
import y.base.YCursor;
import y.base.YList;

/**
 * Specialized list implementation for instances of type
 * {@link y25.base.LayerEdge LayerEdge}.
 * <p>
 * Iterating over the elements of this list is accomplished by retrieving a
 * <code>LayerEdgeCursor</code> via the method {@link #layerEdges layerEdges}
 * (see {@linkplain LayerEdgeCursor the LayerEdgeCursor documentation} for an
 * example).
 * 
 * @author Werner Jainek
 */
public class LayerEdgeList extends YList {

	/**
	 * Creates an empty layeredge list.
	 */
	public LayerEdgeList() {
		super();
	}

	/**
	 * Creates a list that is initialized with a single layeredge provided.
	 */
	public LayerEdgeList(LayerEdge e) {
		super();
		add(e);
	}

	/**
	 * Creates a list that is initialized with the layeredges provided by the
	 * given array of layeredges.
	 */
	public LayerEdgeList(LayerEdge[] layerEdges) {
		super(layerEdges);
	}

	/**
	 * Creates a list that is initialized with the layeredges provided by the
	 * given <code>LayerEdgeCursor</code> object.
	 */
	public LayerEdgeList(LayerEdgeCursor ec) {
		super(ec);
	}

	/**
	 * Creates a list that is initialized with those layeredges from the given
	 * LayerEdgeCursor object for which the given data provider returns
	 * <code>true</code> upon calling its
	 * {@link y.base.DataProvider#getBool getBool} method.
	 * 
	 * @param ec
	 *            A layeredge cursor providing layeredges that should be added
	 *            to this list.
	 * @param predicate
	 *            A data provider that acts as a inclusion predicate for each
	 *            layeredge accessible by the given layeredge cursor.
	 */
	public LayerEdgeList(LayerEdgeCursor ec, DataProvider predicate) {
		super(ec, predicate);
	}

	/**
	 * Creates a list that is initialized with the elements provided by the
	 * given Iterator object.
	 */
	public LayerEdgeList(Iterator it) {
		super(it);
	}

	/**
	 * Returns the first layeredge in this list, or <code>null</code> when the
	 * list is empty.
	 */
	public LayerEdge firstLayerEdge() {
		return (LayerEdge) first();
	}

	/**
	 * Returns the last layeredge in this list, or <code>null</code> when the
	 * list is empty.
	 */
	public LayerEdge lastLayerEdge() {
		return (LayerEdge) last();
	}

	/**
	 * Returns a layeredge cursor for the layeredges.
	 */
	public LayerEdgeCursor layerEdges() {
		return new LayerEdgeCursorImpl(cursor());
	}

	/**
	 * An implementation of the <code>LayerEdgeCursor</code> interface based
	 * on a <code>YCursor</code>.
	 * 
	 * @author Werner Jainek
	 * 
	 */
	private class LayerEdgeCursorImpl implements LayerEdgeCursor {
		private YCursor cursor;

		public LayerEdgeCursorImpl(YCursor cur) {
			cursor = cur;
		}

		public LayerEdge layerEdge() {
			return (LayerEdge) current();
		}

		public void cyclicNext() {
			next();
			if (!ok())
				toFirst();
		}

		public void cyclicPrev() {
			prev();
			if (!ok())
				toLast();
		}

		public boolean ok() {
			return cursor.ok();
		}

		public void next() {
			cursor.next();
		}

		public void prev() {
			cursor.prev();
		}

		public void toFirst() {
			cursor.toFirst();
		}

		public void toLast() {
			cursor.toLast();
		}

		public Object current() {
			return cursor.current();
		}

		public int size() {
			return cursor.size();
		}
	}

}
