/**
 * y25 - Visualizing Graphs in 2.5 Dimensions
 * 
 * Arbeitsbereich: Paralleles Rechnen Prof. Kaufmann
 * Studienarbeit:  Werner Jainek
 * Betreuung:      Nina Lehmann
 *
 */
package y25.base;

import y.base.DataAcceptor;
import y.base.DataProvider;

/**
 * Provides access to data associated with a layer edge. A layer edge map can be
 * considered as a map that allows only layer edges as keys. The layer edges
 * used as keys for the getter/setter methods must belong to the same layered
 * graph.
 * <p>
 * The values bound to the layer edges are initialized with Java(TM) default
 * values(null, 0, 0.0, false) upon initialization.
 * 
 * @author Werner Jainek
 */
public interface LayerEdgeMap extends DataProvider, DataAcceptor {

	/**
	 * Removes the value stored for the given layer edge, if present. Returns
	 * the value that was previously stored for the layer edge, or
	 * <code>null</code> if there was no mapping for that layer edge. A
	 * <code>null</code> return can also mean that the map previously
	 * associated <code>null</code> with the given layer edge.
	 */
	public Object remove(Object layerEdge);

	/**
	 * Returns the value bound to the given layer edge. The value must have been
	 * assiciated to the given layer edge by a call to
	 * {@link LayerEdgeMap#set(Object, Object) set}.
	 * <p>
	 * Returns <code>null</code> if the map contains no mapping for this key.
	 * A return value of <code>null</code> does not necessarily indicate that
	 * the map contains no mapping for the key; it is also possible that the map
	 * explicitly maps the key to <code>null</code>.
	 */
	public Object get(Object layerEdge);

	/**
	 * Returns the boolean value bound to the given layer edge. The value must
	 * have been assiciated to the given layer edge by a call to
	 * {@link LayerEdgeMap#setBool(Object, boolean) setBool}.
	 * <p>
	 * Returns <code>false</code> if the map contains no mapping for this key.
	 * A return value of <code>false</code> does not necessarily indicate that
	 * the map contains no mapping for the key; it is also possible that the map
	 * explicitly maps the key to <code>false</code>.
	 */
	public boolean getBool(Object layerEdge);

	/**
	 * Returns the double value bound to the given layer edge. The value must
	 * have been assiciated to the given layer edge by a call to
	 * {@link LayerEdgeMap#setDouble(Object, double) setDouble}.
	 * <p>
	 * Returns <code>0.0</code> if the map contains no mapping for this key. A
	 * return value of <code>0.0</code> does not necessarily indicate that the
	 * map contains no mapping for the key; it is also possible that the map
	 * explicitly maps the key to <code>0.0</code>.
	 */
	public double getDouble(Object layerEdge);

	/**
	 * Returns the integer value bound to the given layer edge. The value must
	 * have been assiciated to the given layer edge by a call to
	 * {@link LayerEdgeMap#setInt(Object, int) setInt}.
	 * <p>
	 * Returns <code>0</code> if the map contains no mapping for this key. A
	 * return value of <code>0</code> does not necessarily indicate that the
	 * map contains no mapping for the key; it is also possible that the map
	 * explicitly maps the key to <code>0</code>.
	 */
	public int getInt(Object layerEdge);

	/**
	 * Associates the given value to the given layer edge.
	 */
	public void set(Object layerEdge, Object value);

	/**
	 * Associates the given boolean value to the given layer edge.
	 */
	public void setBool(Object layerEdge, boolean value);

	/**
	 * Associates the given double value to the given layer edge.
	 */
	public void setDouble(Object layerEdge, double value);

	/**
	 * Associates the given integer value to the given layer edge.
	 */
	public void setInt(Object layerEdge, int value);
}
