/**
 * y25 - Visualizing Graphs in 2.5 Dimensions (2005)
 * 
 * Arbeitsbereich: Paralleles Rechnen Prof. Kaufmann
 * Studienarbeit:  Werner Jainek
 * Betreuung:      Nina Lehmann
 *
 */

package y25.base;

/**
 * Provides access to data associated with a node. A node map can be considered
 * as a map that allows only nodes as keys. The nodes used as keys for the
 * getter/setter methods must belong to the same layered graph.
 * <p>
 * The values bound to the nodes are initialized with Java(TM) default
 * values(null, 0, 0.0, false) upon initialization.
 * <p>
 * This interface exists primarily because the original interface
 * {@link y.base.NodeMap y.base.NodeMap} only allows nodes that belong to the
 * same graph as keys. This restriction is not posed by this interface. When
 * working with layered graphs, there might be a situation where one NodeMap
 * should store information about nodes from different graphs (wich all belong
 * to the same layered graph) - and this is where this interface comes into
 * play.
 * <p>
 * There is also another reason why this interface exists: orthogonality. The
 * newly defined interfaces {@link GraphMap} and {@link LayerEdgeMap} both have
 * a <code>remove</code> method, which is not true for the original
 * {@link y.base.NodeMap y.base.NodeMap} or
 * {@link y.base.EdgeMap y.base.EdgeMap}.
 * 
 * @author Werner Jainek
 */
public interface NodeMap extends y.base.NodeMap {

	/**
	 * Removes the value stored for the given node, if present. Returns the
	 * value that was previously stored for the graph, or <code>null</code> if
	 * there was no mapping for that node. A <code>null</code> return can also
	 * mean that the map previously associated <code>null</code> with the
	 * given node.
	 */
	public Object remove(Object node);

	/**
	 * Returns the value bound to the given node. The value must have been
	 * assiciated to the given node by a call to
	 * {@link NodeMap#set(Object, Object) set}.
	 * <p>
	 * Returns <code>null</code> if the map contains no mapping for this key.
	 * A return value of <code>null</code> does not necessarily indicate that
	 * the map contains no mapping for the key; it is also possible that the map
	 * explicitly maps the key to <code>null</code>.
	 */
	public Object get(Object node);

	/**
	 * Returns the boolean value bound to the given node. The value must have
	 * been assiciated to the given node by a call to
	 * {@link NodeMap#setBool(Object, boolean) setBool}.
	 * <p>
	 * Returns <code>false</code> if the map contains no mapping for this key.
	 * A return value of <code>false</code> does not necessarily indicate that
	 * the map contains no mapping for the key; it is also possible that the map
	 * explicitly maps the key to <code>false</code>.
	 */
	public boolean getBool(Object node);

	/**
	 * Returns the double value bound to the given node. The value must have
	 * been assiciated to the given node by a call to
	 * {@link NodeMap#setDouble(Object, double) setDouble}.
	 * <p>
	 * Returns <code>0.0</code> if the map contains no mapping for this key. A
	 * return value of <code>0.0</code> does not necessarily indicate that the
	 * map contains no mapping for the key; it is also possible that the map
	 * explicitly maps the key to <code>0.0</code>.
	 */
	public double getDouble(Object node);

	/**
	 * Returns the integer value bound to the given node. The value must have
	 * been assiciated to the given node by a call to
	 * {@link NodeMap#setInt(Object, int) setInt}.
	 * <p>
	 * Returns <code>0</code> if the map contains no mapping for this key. A
	 * return value of <code>0</code> does not necessarily indicate that the
	 * map contains no mapping for the key; it is also possible that the map
	 * explicitly maps the key to <code>0</code>.
	 */
	public int getInt(Object node);

	/**
	 * Associates the given value to the given node.
	 */
	public void set(Object node, Object value);

	/**
	 * Associates the given boolean value to the given node.
	 */
	public void setBool(Object node, boolean value);

	/**
	 * Associates the given double value to the given node.
	 */
	public void setDouble(Object node, double value);

	/**
	 * Associates the given integer value to the given node.
	 */
	public void setInt(Object node, int value);

}
