/**
 * y25 - Visualizing Graphs in 2.5 Dimensions
 * 
 * Arbeitsbereich: Paralleles Rechnen Prof. Kaufmann
 * Studienarbeit:  Werner Jainek
 * Betreuung:      Nina Lehmann
 *
 */
package y25.layout;

/**
 * Partial <code>LayoutStage25D</code> implementation that handles the core
 * layouter part.
 * 
 * This interface mirrors the <code>AbstractLayoutStage</code> interface found
 * in the <code>y.layout</code> package.
 * 
 * @author Werner Jainek
 * 
 */
public abstract class AbstractLayoutStage25D implements LayoutStage25D {

	/**
	 * The core layouter of this layout stage.
	 */
	protected Layouter25D coreLayouter;

	/**
	 * Creates an layout stage without setting the core layouter.
	 */
	public AbstractLayoutStage25D() {

	}

	/**
	 * Initializes this <code>AbstractLayoutStage25D</code> with the given
	 * core layouter.
	 */
	public AbstractLayoutStage25D(Layouter25D core) {
		coreLayouter = core;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see y25.layout.LayoutStage25D#getCoreLayouter25D()
	 */
	public Layouter25D getCoreLayouter25D() {
		return coreLayouter;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see y25.layout.LayoutStage25D#setCoreLayouter25D(y25.layout.Layouter25D)
	 */
	public void setCoreLayouter25D(Layouter25D core) {
		coreLayouter = core;
	}

	/**
	 * Queries the core layouter whether or not the given graph can be layed
	 * out. If the core layouter is not present, <code>true</code> is
	 * returned.
	 */
	protected boolean canLayoutCore(LayeredLayoutGraph llg) {
		if (coreLayouter != null) {
			return coreLayouter.canLayout(llg);
		} else {
			return true;
		}
	}

	/**
	 * Invokes the layout routine of the core layouter. If the core layouter is
	 * not present, the passed layered layout graph will not be modified.
	 */
	protected void doLayoutCore(LayeredLayoutGraph llg) {
		if (coreLayouter != null) {
			coreLayouter.doLayout(llg);
		}
	}

}
