/**
 * y25 - Visualizing Graphs in 2.5 Dimensions
 * 
 * Arbeitsbereich: Paralleles Rechnen Prof. Kaufmann
 * Studienarbeit:  Werner Jainek
 * Betreuung:      Nina Lehmann
 *
 */
package y25.layout;

import y.base.EdgeCursor;
import y.base.NodeCursor;
import y.geom.YPoint;
import y.layout.EdgeLayout;
import y.layout.LayoutGraph;
import y25.base.GraphCursor;
import y25.graphics.BoundingBox;
import y25.graphics.Point3D;

/**
 * A layouter that centers all layers and adjusts their bounding boxes.
 * 
 * @author Werner Jainek
 * 
 */
public class CentralizerLayouter25D implements Layouter25D {

	public boolean canLayout(LayeredLayoutGraph llg) {
		return true;
	}

	public void doLayout(LayeredLayoutGraph llg) {

		// 1. first set all layer realizer to degenerate x/y - coords
		// but leave z value the same
		for (GraphCursor gc = llg.graphs(); gc.ok(); gc.next()) {
			LayerLayout25D ll = (LayerLayout25D) llg.getLayout(gc.graph());
			ll.setRelativeBoundingBox(new BoundingBox());
		}

		// 2. a. calculate the bounding box for the entire graph
		llg.recalculateBoundingBox();
		BoundingBox bb = llg.getBoundingBox();

		// b. center the graphs
		for (GraphCursor gc = llg.graphs(); gc.ok(); gc.next()) {
			double centerX = bb.getCenter().x;
			double centerY = bb.getCenter().y;
			// double rectW2 = (bb.getMax().x - bb.getMin().x)/2.0;
			// double rectH2 = (bb.getMax().y - bb.getMin().y)/2.0;
			moveGraph((LayoutGraph) gc.graph(), -centerX, -centerY);
		}

		// c. recalculate the bb
		llg.recalculateBoundingBox();
		bb = llg.getBoundingBox();

		// 3. set the layer realizers to the same bounds
		BoundingBox relBB = new BoundingBox();
		relBB.include(new Point3D(bb.getMin().x, bb.getMin().y, 0));
		relBB.include(new Point3D(bb.getMax().x, bb.getMax().y, 0));
		for (GraphCursor gc = llg.graphs(); gc.ok(); gc.next()) {
			LayerLayout25D ll = (LayerLayout25D) llg.getLayout(gc.graph());
			ll.setRelativeBoundingBox(relBB);
		}

	}

	/**
	 * Moves all nodes and edges inside a layout graph by the given x/y values. =
	 */
	void moveGraph(LayoutGraph lg, double deltaX, double deltaY) {

		// move all nodes
		for (NodeCursor nc = lg.nodes(); nc.ok(); nc.next()) {
			lg.moveBy(nc.node(), deltaX, deltaY);
		}

		// move all edge points
		for (EdgeCursor ec = lg.edges(); ec.ok(); ec.next()) {
			EdgeLayout el = lg.getLayout(ec.edge());

			for (int i = 0; i < el.pointCount(); ++i) {
				YPoint p = el.getPoint(i);
				el.setPoint(i, p.x + deltaX, p.y + deltaY);
			}
		}
	}
}
