/**
 * y25 - Visualizing Graphs in 2.5 Dimensions
 * 
 * Arbeitsbereich: Paralleles Rechnen Prof. Kaufmann
 * Studienarbeit:  Werner Jainek
 * Betreuung:      Nina Lehmann
 *
 */
package y25.view;

import java.awt.Color;

import javax.media.opengl.GLAutoDrawable;

import y.view.Arrow;
import y.view.NodeRealizer;
import y25.base.LayerEdge;
import y25.graphics.Camera;
import y25.layout.LayerEdgeLayout25D;

/**
 * Provides a mechanism for rendering layer edges into an OpenGL context.
 * <p>
 * Before this realizer can be used to render a layer edge, the layer edge has
 * to be bound to this realizer by calling
 * {@link #bindLayerEdge(LayerEdge) bindLayerEdge}. Note that this is done
 * automatically when calling
 * {@link y25.view.Graph25D#setRealizer(LayerEdge, LayerEdgeRealizer25D) Graph25D.setRealizer(LayerEdge, LayerEdgeRealizer25D)}.
 * 
 * @author Werner Jainek
 */
public interface LayerEdgeRealizer25D extends LayerEdgeLayout25D {

	/**
	 * 2D drawing mode for lines.
	 */
	public static final int MODE_2D = 0;

	/**
	 * 3D drawing mode for lines.
	 */
	public static final int MODE_3D = 1;

	/**
	 * Returns the current display mode for drawing lines. Can be either
	 * <code>LayerEdgeRealizer25D.MODE_2D</code> or
	 * <code>LayerEdgeRealizer25D.MODE_3D</code>.
	 */
	public int getLineDisplayMode();

	/**
	 * Sets the current display mode for drawing lines. Can be either
	 * <code>LayerEdgeRealizer25D.MODE_2D</code> or
	 * <code>LayerEdgeRealizer25D.MODE_3D</code>.
	 */
	public void setLineDisplayMode(int mode);

	/**
	 * Creates a copy of this realizer and returns it.
	 */
	public LayerEdgeRealizer25D createCopy();

	/**
	 * Renders the layer edge this realizer is bound to into the given drawable.
	 * If no layer edge is bound to this realizer, nothing happens.
	 * <p>
	 * The passed camera is the one that is used to display this layer edge. It
	 * can be used, for example, to render a layer edge differently depending on
	 * the viewing position, or to display labels that always face the viewer.
	 */
	public void paintGL(GLAutoDrawable drawable, Camera camera);

	/**
	 * Binds this realizer to the passed layer edge.
	 */
	public void bindLayerEdge(LayerEdge le);

	/**
	 * Returns the layer edge that has been bound to this realizer, or
	 * <code>null</code> if no layer edge has been bound yet.
	 */
	public LayerEdge getLayerEdge();

	/**
	 * Sets the color of this layer edge realizer.
	 */
	public void setLineColor(Color color);

	/**
	 * Returns the color of this layer edge realizer.
	 */
	public Color getLineColor();

	/**
	 * Sets the radius of this layer edge realizer.
	 */
	public void setLineRadius(double d);

	/**
	 * Returns the radius of this layer edge realizer.
	 */
	public double getLineRadius();

	/**
	 * Returns <code>true</code> if this realizer is currently selected,
	 * <code>false</code> otherwise.
	 */
	public boolean isSelected();

	/**
	 * Sets the selection state of this realizer.
	 */
	public void setSelected(boolean selected);

	/**
	 * Returns the arrow at the target of the layer edge. Same as
	 * <code>getTargetArrow</code>.
	 */
	public Arrow getArrow();

	/**
	 * Returns the arrow at the source of the layer edge;
	 */
	public Arrow getSourceArrow();

	/**
	 * Returns the arrow at the target of the layer edge;
	 */
	public Arrow getTargetArrow();

	/**
	 * Sets the arrow at the target of the layer edge. Same as
	 * <code>setTargetArrow</code>.
	 */
	public void setArrow(Arrow arrow);

	/**
	 * Sets the arrow at the source of the layer edge.
	 */
	public void setSourceArrow(Arrow arrow);

	/**
	 * Sets the arrow at the target of the layer edge.
	 */
	public void setTargetArrow(Arrow arrow);

	/**
	 * Returns the realizer of the source node or <code>null</code>, if not
	 * bound to a layer edge yet.
	 */
	public NodeRealizer getSourceRealizer();

	/**
	 * Returns the realizer of the target node or <code>null</code>, if not
	 * bound to a layer edge yet.
	 */
	public NodeRealizer getTargetRealizer();
}
