/**
 * y25 - Visualizing Graphs in 2.5 Dimensions
 * 
 * Arbeitsbereich: Paralleles Rechnen Prof. Kaufmann
 * Studienarbeit:  Werner Jainek
 * Betreuung:      Nina Lehmann
 *
 */
package y25.view;

import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.event.MouseInputListener;

/**
 * Handles mouse events coming from a <code>Graph25DView</code>.
 * <p>
 * A view mode has to be bound to a <code>Graph25DView</code> by calling
 * {@link #setActiveView setActiveView} before it receives any mouse input
 * events. When a view mode is added to a <code>Graph25DView</code> via a call
 * to
 * {@link Graph25DView#addViewMode25D(ViewMode25D) Graph25DView#addViewMode25D},
 * this happens automatically.
 * <p>
 * As a convenience, this class stores the mouse position of an event relative
 * to the mouse position of the last event inside <code>deltaDrag</code>,
 * which can be read inside the event handling methods.
 * <p>
 * When deriving from this class, make sure that all overriden methods you
 * implement first call the respective super method first.
 * 
 * @author Werner Jainek
 * 
 */
public class ViewMode25D implements MouseInputListener, MouseWheelListener {

	/**
	 * The view this mode gets mouse events from.
	 */
	protected Graph25DView view;

	/**
	 * The last mouse click event.
	 */
	protected MouseEvent lastClickEvent;

	/**
	 * The last mouse press event.
	 */
	protected MouseEvent lastPressEvent;

	/**
	 * The last mouse release event.
	 */
	protected MouseEvent lastReleaseEvent;

	/**
	 * The last mouse drag event.
	 */
	protected MouseEvent lastDragEvent;

	/**
	 * The mouse position relative to the last drag event.
	 */
	protected Point deltaDrag;

	/**
	 * Returns the difference vector from <code>from</code> to <code>to</code>.
	 */
	protected Point delta(MouseEvent from, MouseEvent to) {
		return new Point(to.getX() - from.getX(), to.getY() - from.getY());
	}

	/**
	 * Sets the view this mode receives mouse events from. This method has to be
	 * called before any events are passed to this view mode.
	 */
	public void setActiveView(Graph25DView v) {
		view = v;
	}

	// ----- L O W L E V E L M O U S E E V E N T S -----

	public void mouseClicked(MouseEvent e) {
		lastClickEvent = e;
	}

	public void mousePressed(MouseEvent e) {
		lastDragEvent = e;
		lastPressEvent = e;
	}

	public void mouseReleased(MouseEvent e) {
		lastReleaseEvent = e;
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mouseDragged(MouseEvent e) {
		deltaDrag = delta(lastDragEvent, e);
		lastDragEvent = e;

		switch (e.getButton()) {
		case MouseEvent.BUTTON1:
			if (shiftPressed(e))
				mouseShiftDraggedLeft(e);
			else
				mouseDraggedLeft(e);
			break;

		case MouseEvent.BUTTON2:
			if (shiftPressed(e))
				mouseShiftDraggedMiddle(e);
			else
				mouseDraggedMiddle(e);
			break;

		case MouseEvent.BUTTON3:
			if (shiftPressed(e))
				mouseShiftDraggedRight(e);
			else
				mouseDraggedRight(e);
			break;

		default:
			break;
		}
	}

	public void mouseMoved(MouseEvent e) {
	}

	public void mouseWheelMoved(MouseWheelEvent e) {
	}

	// ----- H I G H L E V E L M O U S E E V E N T S -----
	public void mouseDraggedLeft(MouseEvent e) {
	}

	public void mouseDraggedMiddle(MouseEvent e) {
	}

	public void mouseDraggedRight(MouseEvent e) {
	}

	public void mouseShiftDraggedLeft(MouseEvent e) {
		mouseDraggedLeft(e);
	}

	public void mouseShiftDraggedMiddle(MouseEvent e) {
		mouseDraggedMiddle(e);
	}

	public void mouseShiftDraggedRight(MouseEvent e) {
		mouseDraggedRight(e);
	}

	/**
	 * Determines if the SHIFT modifier key has been pressed.
	 */
	protected boolean shiftPressed(MouseEvent e) {
		int onmask = MouseEvent.SHIFT_DOWN_MASK;
		return (e.getModifiersEx() & onmask) == onmask;
	}

}
