/**
 * y25 - Visualizing Graphs in 2.5 Dimensions
 * 
 * Arbeitsbereich: Paralleles Rechnen Prof. Kaufmann
 * Studienarbeit:  Werner Jainek
 * Betreuung:      Nina Lehmann
 *
 */
package y25.view.realizer;

import javax.media.opengl.GLAutoDrawable;

import y.view.Arrow;
import y25.graphics.Camera;
import y25.graphics.Graphics3D;
import y25.graphics.Point3D;
import y25.graphics.Vector3D;

/**
 * Paints <code>Arrow</code>s into an OpenGL context.
 * 
 * @author Werner Jainek
 */
public class ArrowPainter {

	/**
	 * The number of sides for the painted arrows.
	 */
	public static int standardArrowSides = 10;

	/**
	 * Paints the given <code>arrow</code> into the given OpenGL context.
	 * 
	 * @param drawable
	 *            the drawable the drawing operations go to.
	 * @param camera
	 *            the camera that is used to view the scene.
	 * @param arrow
	 *            the arrow that has to be painted.
	 * @param tip
	 *            the position of the tip of the arrow.
	 * @param tipToBase
	 *            a vector from the tip to the base of the arrow.
	 * @param radius
	 *            the radius of the arrow base.
	 */
	public static void paintGL(GLAutoDrawable drawable, Camera camera,
			Arrow arrow, Point3D tip, Vector3D tipToBase, float radius) {

		if (!tipToBase.isZero()) {

			switch (arrow.getType()) {
			case Arrow.NONE_TYPE:
				break;

			default:
			case Arrow.STANDARD_TYPE:
				Graphics3D.drawCone(drawable, tip,
						tip.getTranslated(tipToBase), radius,
						standardArrowSides, true);
				break;

			}// switch

		}// if

	}
}
