/**
 * y25 - Visualizing Graphs in 2.5 Dimensions
 * 
 * Arbeitsbereich: Paralleles Rechnen Prof. Kaufmann
 * Studienarbeit:  Werner Jainek
 * Betreuung:      Nina Lehmann
 *
 */
package y25.view.realizer;

import java.awt.Color;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;

import y.view.YLabel;
import y25.graphics.Camera;
import y25.graphics.Point3D;

import com.sun.opengl.util.GLUT;

/**
 * Paints <code>YLabel</code>s into an OpenGL context.
 * 
 * @author Werner Jainek
 */
public class LabelPainter {

	/**
	 * Renders the passed label into the given drawable.
	 * 
	 * @param drawable
	 *            the drawable that the label will be rendered into.
	 * @param camera
	 *            used to determine the orientation of the label such that is
	 *            faces the viewer.
	 * @param label
	 *            the label that will be rendered.
	 * @param pos
	 *            the position of the labbel. The text of the label will be
	 *            centered at this position.
	 */
	public static void paintGL(GLAutoDrawable drawable, Camera camera,
			YLabel label, Point3D pos) {
		GL gl = drawable.getGL();
		GLUT glut = new GLUT();

		// retrieve raster coordinates of position
		Point3D rasterPos = camera.getHUDCoordinates(drawable, pos);

		if (rasterPos != null) {

			// Leads to same x/y coordinates, but z-values not correct as they
			// depend on device
			// Point3D rasterPos2 = camera.getScreenCoordinates(pos);
			// rasterPos.x = rasterPos2.x;
			// rasterPos.y = rasterPos2.y;

			// activate "head-up-display"
			camera.activateHUD(drawable);

			// specify the material
			gl.glDisable(GL.GL_COLOR_MATERIAL);
			Color textColor = label.getTextColor();
			float mat_none[] = { 0.0f, 0.0f, 0.0f, 1.0f };
			float mat_diffuse[] = { ((float) textColor.getRed()) / 255.0f,
					((float) textColor.getGreen()) / 255.0f,
					((float) textColor.getBlue()) / 255.0f, 1.0f };
			float mat_noshininess[] = { 0.0f };
			gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE,
					mat_diffuse, 0);
			gl.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR, mat_none, 0);
			gl.glMaterialfv(GL.GL_FRONT, GL.GL_SHININESS, mat_noshininess, 0);
			gl.glMaterialfv(GL.GL_FRONT, GL.GL_EMISSION, mat_none, 0);

			// draw the text
			String labelText = label.getText();

			int font = GLUT.BITMAP_HELVETICA_12;
			int length = glut.glutBitmapLength(font, labelText);
			gl.glRasterPos3f(rasterPos.x - ((float) length / 2.0f),
					rasterPos.y - 4, rasterPos.z);
			glut.glutBitmapString(font, labelText);

			gl.glEnable(GL.GL_COLOR_MATERIAL);

			camera.deactivateHUD(drawable);
		}

	}

}
