/**
 * 
 */
package y25.view.realizer;

import java.awt.Color;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;

import y.view.Arrow;
import y.view.NodeRealizer;
import y25.base.LayerEdge;
import y25.graphics.BoundingBox;
import y25.graphics.Camera;
import y25.graphics.Graphics3D;
import y25.graphics.Point3D;
import y25.graphics.Vector3D;
import y25.view.Graph25D;
import y25.view.LayerEdgeRealizer25D;
import y25.view.NodeRealizer25D;

/**
 * Displays a layer edge as a polygon line.
 * 
 * @author Werner Jainek
 */
public class PolyLineLayerEdgeRealizer25D implements LayerEdgeRealizer25D {

	LayerEdge layerEdge;

	Color color;

	double lineRadius;

	boolean isSelected;

	int lineDisplayMode;

	Arrow sourceArrow;

	Arrow targetArrow;

	/**
	 * The length of the arrow head.
	 */
	static float arrowLength = 10;

	/**
	 * The radius of the arrow head's base.
	 */
	static float arrowRadius = 4;

	/**
	 * Additional scaling of the arrow head.
	 */
	static float arrowScale = 0;

	public PolyLineLayerEdgeRealizer25D() {

		color = new Color(200, 200, 200);
		lineRadius = 1;
		isSelected = false;
		lineDisplayMode = MODE_3D;
		sourceArrow = Arrow.NONE;
		targetArrow = Arrow.NONE;
	}

	public BoundingBox getBoundingBox() {
		BoundingBox bb = new BoundingBox();

		if (layerEdge != null) {
			Graph25D g25d = (Graph25D) layerEdge.getLayeredGraph();
			Point3D src = g25d.getPoint3D(layerEdge.source());
			Point3D trg = g25d.getPoint3D(layerEdge.target());

			bb.include(src);
			bb.include(trg);
		}

		return bb;
	}

	public LayerEdgeRealizer25D createCopy() {

		PolyLineLayerEdgeRealizer25D ler = new PolyLineLayerEdgeRealizer25D();
		ler.setLineColor(color);
		ler.setLineRadius(lineRadius);
		ler.setLineDisplayMode(lineDisplayMode);
		ler.setSourceArrow(sourceArrow);
		ler.setTargetArrow(targetArrow);
		return ler;
	}

	public void bindLayerEdge(LayerEdge le) {
		layerEdge = le;

	}

	public LayerEdge getLayerEdge() {
		return layerEdge;
	}

	public void setLineColor(Color color) {
		this.color = new Color(color.getRGB());
	}

	public Color getLineColor() {
		return color;
	}

	public void setLineRadius(double d) {
		lineRadius = d;

	}

	public double getLineRadius() {
		return lineRadius;
	}

	/**
	 * Returns <code>true</code> if this realizer is currently selected,
	 * <code>false</code> otherwise.
	 */
	public boolean isSelected() {
		return isSelected;
	}

	/**
	 * Sets the selection state of this realizer.
	 */
	public void setSelected(boolean selected) {
		isSelected = selected;
	}

	/**
	 * @see y25.view.LayerEdgeRealizer25D#getLineDisplayMode()
	 */
	public int getLineDisplayMode() {
		return lineDisplayMode;
	}

	/**
	 * @see y25.view.LayerEdgeRealizer25D#setLineDisplayMode(int)
	 */
	public void setLineDisplayMode(int mode) {
		lineDisplayMode = mode;
	}

	public Arrow getArrow() {
		return getTargetArrow();
	}

	public Arrow getSourceArrow() {
		return sourceArrow;
	}

	public Arrow getTargetArrow() {
		return targetArrow;
	}

	public void setArrow(Arrow arrow) {
		setTargetArrow(arrow);
	}

	public void setSourceArrow(Arrow arrow) {
		sourceArrow = arrow;
	}

	public void setTargetArrow(Arrow arrow) {
		targetArrow = arrow;
	}

	public NodeRealizer getSourceRealizer() {
		if (layerEdge == null) {
			return null;
		}

		Graph25D g25d = (Graph25D) layerEdge.getLayeredGraph();
		return g25d.getRealizer(layerEdge.source());
	}

	public NodeRealizer getTargetRealizer() {
		if (layerEdge == null) {
			return null;
		}

		Graph25D g25d = (Graph25D) layerEdge.getLayeredGraph();
		return g25d.getRealizer(layerEdge.target());
	}

	protected void setCylinderMaterial(GLAutoDrawable drawable) {
		GL gl = drawable.getGL();

		gl.glDisable(GL.GL_COLOR_MATERIAL);
		Color nodeColor = getLineColor();
		float mat_none[] = { 0.0f, 0.0f, 0.0f, 1.0f };
		float mat_diffuse[] = { ((float) nodeColor.getRed()) / 255.0f,
				((float) nodeColor.getGreen()) / 255.0f,
				((float) nodeColor.getBlue()) / 255.0f, 1.0f };
		float mat_noshininess[] = { 0.0f };
		gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, mat_diffuse, 0);
		gl.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR, mat_none, 0);
		gl.glMaterialfv(GL.GL_FRONT, GL.GL_SHININESS, mat_noshininess, 0);
		gl.glMaterialfv(GL.GL_FRONT, GL.GL_EMISSION, mat_none, 0);
	}

	protected void unsetCylinderMaterial(GLAutoDrawable drawable) {
		GL gl = drawable.getGL();
		gl.glEnable(GL.GL_COLOR_MATERIAL);
	}

	public void paintGL(GLAutoDrawable drawable, Camera camera) {

		if (layerEdge != null) {
			GL gl = drawable.getGL();

			setCylinderMaterial(drawable);

			// get the source and target coordinates
			Graph25D g25d = (Graph25D) layerEdge.getLayeredGraph();
			Point3D src = g25d.getPoint3D(layerEdge.source());
			Point3D trg = g25d.getPoint3D(layerEdge.target());

			// draw the line
			if (lineDisplayMode == MODE_3D) {
				Graphics3D.drawCylinder(drawable, src, trg, (float) lineRadius,
						10, true);
			} else {
				gl.glLineWidth((float) getLineRadius() * 2);
				gl.glBegin(GL.GL_LINE_STRIP);
				gl.glVertex3f((float) src.x, (float) src.y, (float) -src.z);
				gl.glVertex3f((float) trg.x, (float) trg.y, (float) -trg.z);
				gl.glEnd();
			}

			// determine arrow positions
			NodeRealizer snr = getSourceRealizer();
			NodeRealizer tnr = getTargetRealizer();

			Vector3D sourceArrowTipToBase = new Vector3D(src, trg).normalize();
			Point3D sourceArrowTip = ((NodeRealizer25D) snr).intersect(trg,
					src, src.z);

			Vector3D targetArrowTipToBase = new Vector3D(trg, src).normalize();
			Point3D targetArrowTip = ((NodeRealizer25D) tnr).intersect(src,
					trg, trg.z);

			// draw arrows
			if (sourceArrowTip != null)
				ArrowPainter.paintGL(drawable, camera, getSourceArrow(),
						sourceArrowTip, sourceArrowTipToBase.scale(arrowLength
								+ (float) lineRadius * 2 * arrowScale),
						arrowRadius);
			if (targetArrowTip != null)
				ArrowPainter.paintGL(drawable, camera, getTargetArrow(),
						targetArrowTip, targetArrowTipToBase.scale(arrowLength
								+ (float) lineRadius * 2 * arrowScale),
						arrowRadius);

			unsetCylinderMaterial(drawable);
		}

	}

}
