/**
 * y25 - Visualizing Graphs in 2.5 Dimensions
 * 
 * Arbeitsbereich: Paralleles Rechnen Prof. Kaufmann
 * Studienarbeit:  Werner Jainek
 * Betreuung:      Nina Lehmann
 *
 */
package y25.view.viewmodes;

import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;

import y25.graphics.Point3D;
import y25.view.ViewMode25D;

/**
 * Provides navigation inside a <code>Graph25DView</code> by changing its
 * camera parameters.
 * <p>
 * Here is how to navigate when using this view mode:
 * <ul>
 * <li>Rotation - Click left mouse button and drag.</li>
 * <li>Zoom - Click middle mouse button and drag or use scrollwheel.</li>
 * <li>Change Perspective - Hold Shift, click middle mouse button and drag.</li>
 * <li>Move Up/down - Click right mouse button and drag.</li>
 * </ul>
 * 
 * @author Werner Jainek
 * 
 */
public class NavigationMode25D extends ViewMode25D {

	// LEFT -> ROTATE
	public void mouseDraggedLeft(MouseEvent e) {

		if (view != null) {
			view.getCamera().rotate(-(float) deltaDrag.y, 0.0f,
					(float) deltaDrag.x);
		}
	}

	// MIDDLE -> ZOOM
	public void mouseDraggedMiddle(MouseEvent e) {

		if (view != null) {
			view.getCamera().zoom((float) deltaDrag.y / 100.0f);
		}
	}

	// MIDDLE / SHIFT -> PERSPECTIVE
	public void mouseShiftDraggedMiddle(MouseEvent e) {

		if (view != null) {
			view.getCamera().changePerspective((float) deltaDrag.y);
		}
	}

	// WHEEL -> ZOOM
	public void mouseWheelMoved(MouseWheelEvent e) {

		if (view != null) {
			view.getCamera().zoom((float) e.getWheelRotation() / 100.0f);
		}
	}

	// RIGHT -> POSITION Z
	public void mouseDraggedRight(MouseEvent e) {

		if (view != null) {
			Point3D target = view.getCamera().getTarget();
			target.z += (float) deltaDrag.y;
			view.getCamera().setTarget(target);

		}
	}

}

//
// // RIGHT -> POSITION X/Y
// public void mouseDraggedRight(MouseEvent e) {
//	
// if(view != null) {
// Point3D target = view.getCamera().getTarget();
//		
// // construct the coordinate system for the move-vector
// Vector3D viewDir = view.getCamera().getViewDirection();
// Vector3D upDir = view.getCamera().getUpDirection();
//		
// Vector3D right = viewDir.getCrossed(upDir);
// Vector3D front = new Vector3D(0,0,1).cross(right).normalize();
//		
//		
// // Point3D position = view.getCamera().getPosition();
// // System.out.println("Pos " + position.toString());
// // System.out.println("Trg " + target.toString());
// // System.out.println("View " + viewDir.toString());
// // System.out.println("UP " + upDir.toString());
// // System.out.println("Right" + right.toString() );
// // System.out.println("Front" + front.toString());
//		
//		
// target.translate( right.scale( -(float)deltaDrag.x ) );
// target.translate( front.scale( -(float)deltaDrag.y ) );
//		
// //target.getTranslated( new Vector3D( (float)deltaDrag.x * 10.0f ,0,0 ) );
// //target.getTranslated( new Vector3D( 0, (float)deltaDrag.y * 10.0f ,0 ) );
//		
// view.getCamera().setTarget( target );
// //view.getCamera().setTarget(100,100,0);
// }
// }
